//Video Filters Handling Functions 

import flash.filters.*;

import lib.FilterEvents.DispMapFilterEventClass;
import lib.FilterEvents.VideoFilterEventClass;
import lib.FilterEvents.AsciiFilterEvent;
import lib.FilterEvents.BlendFilterEvent;
import lib.FilterEvents.ShaderFilterEvent;

import mx.events.DragEvent;
import mx.events.ModuleEvent;
import mx.managers.DragManager;
import mx.modules.ModuleLoader;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.http.mxml.HTTPService;

//Playlist XML file
public var modulelisturl:String="modules/moduleList.xml";

[Bindable]
private var modulelist:XMLList;
[Bindable]
private var filtersList1:XMLList = new XMLList;
[Bindable]
private var filtersList2:XMLList = new XMLList;

private var newSelectedIndex:int;
private var Hue:Number=-180;

private var curImdex:int;
private var targetColors:Array = ["0x15b711","0x000000"];
private var normalColors:Array = ["0x000000","0x000000"];

  
 private function init_filters():void
 {
  load_moduleList();
 }
  
  
//Loads the XML module list using HTTP  
private function load_moduleList():void
{
  	var modulelistService:HTTPService = new HTTPService();
  	modulelistService.url=modulelisturl;
  	modulelistService.resultFormat="e4x";
  	modulelistService.showBusyCursor=true;
  	modulelistService.addEventListener(ResultEvent.RESULT, onModuleListResult);
    modulelistService.addEventListener(FaultEvent.FAULT, onModFault);
    modulelistService.send();	
}

//Creates the module list  from the xml results 
private function onModuleListResult(event:ResultEvent):void
{
  	trace(event.result);
    var resultXML:XML= new XML(event.result);
  	modulelist = new XMLList();
	modulelist=resultXML.category;
}
  
//HTTP request fault
private function onModFault(event:FaultEvent):void
{
  trace(event.fault);
}
  
/////////////////////////////////////////////////////////////////
  
 //Drag n Drop functions for filters ComboBoxes
 private function onStartDrag():void
 {
 	this.filtersCombo1.setStyle("fillColors", targetColors); 
 	this.filtersCombo2.setStyle("fillColors", targetColors); 
 	this.filterTipLabel1.visible = false;
 	this.filterTipLabel2.visible = false;
 }
 
 
 
 private function dragEnterHandler(event:DragEvent):void
 {
 	var dropTarget:ComboBox = event.currentTarget as ComboBox;
 	dropTarget.setStyle("borderThickness", 5);
 	dropTarget.setStyle("fillColors", targetColors); 
 	DragManager.acceptDragDrop(dropTarget);
 }
 
 private function dragExitHandler(event:DragEvent):void
 {
 	var dropTarget:ComboBox = event.currentTarget as ComboBox;
 }
 
 //Add filter to video  1
 private function dragDropHandler1(event:DragEvent):void
 {
	this.filterTipLabel2.visible = false;
 	var filter:Array = event.dragSource.dataForFormat("treeItems") as Array;
 	var node:XML = XML(filter[0]);
 	if(node.name().localName == "filter")
 	 {
 	  	if(filtersList1== null)
 	  			filtersList1 = new XMLList();	
 	  	filtersList1 += filter[0];
 	  	var url:String = filter[0].@module;
 	  	loadModule1(url);
 	 } 
 	revertColors();
 }
 
//Add filter to video  2
private function dragDropHandler2(event:DragEvent):void
 {	
 	var filter:Array = event.dragSource.dataForFormat("treeItems") as Array;
 	var node:XML = XML(filter[0]);
 	if(node.name().localName == "filter")
 	 {
 	 	if(filtersList2== null)
 	 		filtersList2 = new XMLList();	
 	  	filtersList2 += filter[0];
 	  	var url:String = filter[0].@module;
 	  	loadModule2(url);
 	 } 
 	revertColors();
 }
 
 private function revertColors():void
 {
 	this.filtersCombo1.setStyle("fillColors", normalColors);
 	this.filtersCombo2.setStyle("fillColors", normalColors);
 }
 
 ////////////////////////////////////////////////////////////////////////////////////
 //Load module (executed when the  user drops a filter to combo box
 
 private function loadModule1(url:String):void
 {
 	var ModLoad:ModuleLoader = new ModuleLoader();
 	ModLoad.x =0;
 	ModLoad.y =0;
 	filterCanvas1.addChild(ModLoad);
 	ModLoad.width = filterCanvas1.width;
 	ModLoad.height = filterCanvas1.height;
 	ModLoad.url = url;
 	ModLoad.addEventListener(ModuleEvent.READY,readyHandler1); 	
 }
 
 private function loadModule2(url:String):void
 {
 	var ModLoad:ModuleLoader = new ModuleLoader();
 	filterCanvas2.addChild(ModLoad);
 	ModLoad.x =0;
 	ModLoad.y =0;
 	ModLoad.width = filterCanvas2.width;
 	ModLoad.height = filterCanvas2.height;
 	ModLoad.url = url;
 	ModLoad.addEventListener(ModuleEvent.READY,readyHandler2);	
 } 
 
 //Activate module when finished loading
 private function readyHandler1(event:ModuleEvent):void
 {
 	//Add event listener for video filter events according to type
 	var ModLoad:ModuleLoader = event.currentTarget as ModuleLoader
 	var ModInt:Object = ModLoad.child as Object;
 	var type:String = this.filtersList1[filtersList1.length()-1].@type;
 	var label:String = this.filtersList1[filtersList1.length()-1].@label;
 	switch (type)
 	{
 		case "convolution":
 			ModInt.addEventListener("VideoFilterEvent", convfilterEvntHandler1);
 			break;
 		case "color":
 			ModInt.addEventListener("VideoFilterEvent", colorfilterEvntHandler1);
 			break;
 		case "noise":
 			ModInt.addEventListener("DispMapFilterEvent", noisefilterEvntHandler1);
 			break;	
 		case "shader":
 			ModInt.addEventListener("ShaderFilterEvent", shaderEvntHandler1);
 			break;				
 		case "ascii":
 			ModInt.addEventListener("AsciiFilterEvent", asciiEventHandler1);	
 			break;
 		case "blend":
 			ModInt.addEventListener("BlendFilterEvent", blendEventHandler1);
 			break;					
 	}
 	this.filtersCombo1.selectedIndex =  filtersList1.length()-1;
 }
 
 private function readyHandler2(event:ModuleEvent):void
 {
 	//Add event listener for video filter events according to type
 	var ModLoad:ModuleLoader = event.currentTarget as ModuleLoader
 	var ModInt:Object = ModLoad.child as Object;
 	var type:String = this.filtersList2[filtersList2.length()-1].@type;
 	var label:String = this.filtersList2[filtersList2.length()-1].@label;
 	switch (type)
 	{
 		case "convolution":
 			ModInt.addEventListener("VideoFilterEvent", convfilterEvntHandler2);
 			break;
 		case "color":
 			ModInt.addEventListener("VideoFilterEvent", colorfilterEvntHandler2);
 			break;
 		case "noise":
 			ModInt.addEventListener("DispMapFilterEvent", noisefilterEvntHandler2);
 			break;	
  		case "shader":
 			ModInt.addEventListener("ShaderFilterEvent", shaderEvntHandler2);
 			break;													
 		case "ascii":
 			ModInt.addEventListener("AsciiFilterEvent", asciiEventHandler2);		
 			break;
 		case "blend":
 		 	ModInt.addEventListener("BlendFilterEvent", blendEventHandler2);
 			break;			
 	}
 	this.filtersCombo2.selectedIndex =  filtersList2.length()-1; 
 }
 
private function activateModule1():void
{
	var indx:int = filtersCombo1.selectedIndex;
	var count:int = filterCanvas1.numChildren;
	var ModLoad:ModuleLoader;
	for (var i:int = 0; i < count; i++)
	{
		ModLoad = filterCanvas1.getChildAt(i) as ModuleLoader;
		if(i!=indx)
		  ModLoad.visible = false; 
		else		 	
		   ModLoad.visible = true;
	}
} 

private function activateModule2():void
{
	var indx:int = filtersCombo2.selectedIndex;
	var count:int = filterCanvas2.numChildren;
	var ModLoad:ModuleLoader;
	for (var i:int = 0; i < count; i++)
	{
		ModLoad = filterCanvas2.getChildAt(i) as ModuleLoader;
		if(i!=indx)
		  ModLoad.visible = false; 
		else
		   ModLoad.visible = true; 
	}
} 


 
 //////////////////////////////////////////////////////////////////////////////////////
 //Delete a filter from filters Comboboxes
 private function deleteFilter(ctrl:uint):void
 {
 	var selectedFilter:int;
 	var ModLoad:ModuleLoader;
 	var ModInt:Object;
 	var vid_filters:Array;
 	if(ctrl==1)
 	{
      // Save the currently selected index.
      selectedFilter = filtersCombo1.selectedIndex;
      
      ModLoad = filterCanvas1.getChildAt(selectedFilter) as ModuleLoader;
      ModInt = ModLoad.child as Object;
      
      //If filter has been enabled, delete it from filter list
      if(ModInt.filter_enabled == true)
      {
      	if(filtersCombo1.selectedLabel == "ASCII Art")
      		//Disable ascii effect
      		this.ascii1Disable();
      	else if (filtersCombo1.selectedLabel == "Blend Images")
      		//Disable blend
      		this.blend1Disable();	
      	else
      	{
      	vid_filters = vid1Bmp.filters;
      	vid_filters.splice(selectedFilter,1);
      	vid1Bmp.filters = vid_filters;
      	if(ModInt.hasOwnProperty("time"))
      	  ModInt.time.stop();      		
      	}
      } 
      
      //Unload module and remove from canvas
      ModLoad.unloadModule();                                          
      filterCanvas1.removeChildAt(selectedFilter);
       
      
      // Delete the currently selected filter.
      delete (filtersList1[selectedFilter]);
      
      // Reselect the next logical item in the combo box.
      newSelectedIndex = (selectedFilter==0) ? 0 : selectedFilter - 1;
  
     // Change the selected index of the data grid at a later frame.
     callLater ( changeComboBox1Index );
   }
  else if(ctrl==2)
   {
   	  // Save the currently selected index.
      selectedFilter = filtersCombo2.selectedIndex;
      
      
       ModLoad = filterCanvas2.getChildAt(selectedFilter) as ModuleLoader;
       ModInt = ModLoad.child as Object;

      
      //If filter has been enabled, delete it from filter list
      if(ModInt.filter_enabled == true)
      {
      	if(filtersCombo2.selectedLabel == "ASCII Art")
      		//Disable ascii filter
      		this.ascii2Disable();
      	else if (filtersCombo2.selectedLabel == "Blend Images")
      		//Disable blend filter
      		this.blend2Disable();
      	else
      	{
	      	vid_filters = vid2Bmp.filters;
	      	vid_filters.splice(selectedFilter,1);
	      	vid2Bmp.filters = vid_filters;
	      	if(ModInt.hasOwnProperty("time"))
	      	  ModInt.time.stop();
      	}  
      } 
      
      //Unload module and remove from canvas
      ModLoad.unloadModule();                                          
      filterCanvas2.removeChildAt(selectedFilter);
                                                
      // Delete the currently selected filter.
      delete (filtersList2[selectedFilter]);
                
     // Reselect the next logical item in the combo box.
      newSelectedIndex = (selectedFilter==0) ? 0 : selectedFilter - 1;

     // Change the selected index of the data grid at a later frame.
     callLater ( changeComboBox2Index );
   }
             
 }    
 
 // This is a workaround for a known issue with
// List-based components where deleting an item 
// from the control's dataProvider leaves the 
// selectedIndex at an incorrect value. The workaround
// is to reassign the data provider at least a 
// frame later and to change the index there. 
 private function changeComboBox1Index():void
{
	
   // Reassign the data grid's data provider.
   filtersCombo1.dataProvider = filtersList1;

   // Set the selected index.
   filtersCombo1.selectedIndex = newSelectedIndex;
   
   //Activate module
   activateModule1();
   
   //Workaround 2
   //Erasing a filter from the ComboBox results
   //the filter to be erased also from the Tree List
   //Reload the moduleList
   load_moduleList();
  	
}


 private function changeComboBox2Index():void
 { 	
   // Reassign the data grid's data provider.
   filtersCombo2.dataProvider = filtersList2;

   // Set the selected index.
   filtersCombo2.selectedIndex = newSelectedIndex;
   
   //Activate module
   activateModule2();
   
   //Workaround 2
   //Erasing a filter from the ComboBox results
   //the filter to be erased also from the Tree List
   //Reload the moduleList
   load_moduleList();
}

 

//Delete all filters
private function deleteAll1():void
{
 var i:int;
 var ModLoad:ModuleLoader;
 var ModInt:Object;
 for(i=0;i<filterCanvas1.numChildren;i++)
  {
  	ModLoad = filterCanvas1.getChildAt(i) as ModuleLoader;
  	ModInt = ModLoad.child as Object;
  	if(ModInt.hasOwnProperty("time"))
      ModInt.time.stop();
  	ModLoad.unloadModule();
  	delete (filtersList1[i]);
  }	
  filterCanvas1.removeAllChildren();
  filtersList1 = null;
  vid1Bmp.filters = null;
  load_moduleList();
  filtersCombo1.dataProvider = filtersList1;
  
  //Disable other filters
  //ASCII Filter
  if(this._ascii1Enabled)
  	this.ascii1Disable();
  //BLEND FILTER
   if(this._blend1Enabled)
  	this.blend1Disable();
  
}

private function deleteAll2():void
{
 var i:int;
 var ModLoad:ModuleLoader;
 var ModInt:Object;
 for(i=0;i<filterCanvas2.numChildren;i++)
  {
  	ModLoad = filterCanvas2.getChildAt(i) as ModuleLoader;
  	ModInt = ModLoad.child as Object;
  	if(ModInt.hasOwnProperty("time"))
      ModInt.time.stop();
  	ModLoad.unloadModule();
    delete (filtersList2[i]);   
  }	
  filterCanvas2.removeAllChildren();
  filtersList2 = null;
  vid2Bmp.filters = null;
  load_moduleList();
  filtersCombo2.dataProvider = filtersList2;
  //Disable other filters
  //ASCII FILTER
  if(this._ascii2Enabled)
   this.ascii2Disable();
  //BLEND FILTER
  if(this._blend2Enabled)
  	this.blend2Disable();
}


